#!/usr/bin/perl -w
###############################################################################
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
###############################################################################

###############################################################################
# Author: Robby Garrison, Vulcan Technologies
#         robby@vulcantechnologies.com
#	  
# Description:
#   Parses the Dan's Guardian access.log file for *DENIED* entries,
#   and outputs them to HTML.  Also rotates it's own output files, and
#   uses much cheapness to try to keep the access.log file rotated
#   too.
#
# To Use:
#   Change the "$log_file" and "$parser_output_dir" variables to match
#   your setup.  To run, simply execute './dg-log-parser-0.03.pl' (you may
#   need to 'chmod +x dg-log-parser-0.03.pl') and then
#   cd to the  "$parser_output_dir" and use a web browser to open the 
#   *.html file with the appropriate date name, and view your log output
#   *NOTE*  this log-parser only outputs log entries where access to a 
#   page has been denied (since these are the only entries of interest to
#   me 8-), but you can change this behavior by modifying the regex under
#   "#Begin parsing of log data".
###############################################################################
$sourcedir="";
$destdir="";
($sourcedir,$destdir)=@ARGV;
#    ($WYear,$wMon,$WDay) = @ARGV unless defined $WYear;
if (length($sourcedir)==0||length($destdir)==0)
    {print "Usage: index.pl SRC DST\n"; exit}
print "Indexing ${sourcedir} . index.html will be created in ${destdir} \.\n";
# exit;

use File::Copy;

# Open log file for reading
$log_file = '/var/log/dansguardian/access.log';
# $parser_output_dir = '/var/log/dansguardian/html';
$parser_output_dir = $destdir;

# Make sure that the output directory for the parser *.html files exists & create if it doesn't
unless (-e $parser_output_dir && -d $parser_output_dir) {
    mkdir ("$parser_output_dir");
}

open( LOG_FILE, "<", "$log_file" )
    or die "No log file exists.  Is Dan's Guardian even running?\n $! \n";

# Read current time...
($null,$minute,$hour,$day,$month,$year,$null,$null,$null)=localtime(time);

# ...and adjust as necessary
if ($hour < 12) {
    $ampm = "am";
} else {
    $ampm = "pm";
}
if ($hour == 12) {
    $hour=$hour;
} elsif ($hour == 0) {
    $hour = $hour + 12;
} elsif ($hour > 12) {
    $hour = $hour - 12;
}
if ($minute < 10) {
    $minute = "0$minute";
}
$year = $year + 1900;

$month = $month + 1;
@months = qw(January February March April May June July August September October November December);
$written_month = $months[$month - 1];

# Open 'log.html' for writing, and select it to send output to
$html=("$parser_output_dir/$written_month-$day-$year.html");
open( HTML, ">", "$html" );
select HTML;

# Print html-header
print "<html><head><title>Ubuntu CE Parental Control Access Denied Log: $written_month $day\, $year $hour\:$minute$ampm </title>\n";
print "</head><body bgcolor=\"ffffff\"><h2>Ubuntu CE Parental Control Access Denied Log<h2/><h3>Log Date\: $written_month $day\, $year $hour\:$minute$ampm </h3>\n";
print "<table border=\"1\" bgcolor=\"7f7f7f\">\n";
print "<tr bgcolor=000ea8><td><font color=#ffffff size=+1><b>Date</b></font></td>\n";
#print "<td><font color=#ffffff size=+1><b>PC Address</b></font></td>\n";

# Uncomment the following line, the two other appropriate lines further down,
# and the "get_hostname" subroutine to add hostname resolution to the log output file.
# Each of your clients must have entries in your local "/etc/hosts" file, or your DNS server.
# **WARNING** Turning on this option greatly increases the time it takes to generate
# the log ouput file.

#print "<td><font color=#ffffff size=+1><b>PC Name</b></font></td>\n";
print "<td><font color=#ffffff size=+1><b>URL</b></font></td>\n";
print "<td><font color=#ffffff size=+1><b>Reason for Denial</b></font></td></tr>\n";

# init variable used to alternate row color
$tr_alternate = 0;

# Begin parsing of log data
foreach $logfile_line (<LOG_FILE>) {
    $logfile_line =~ s/GET//;
    if ($logfile_line =~ m/\*DENIED\*/) {
	$tr_alternate = $tr_alternate + 1;
	@logfile_fields = split /\s+/, $logfile_line, 6;
	$date = ($logfile_fields[0]);
	$time = ($logfile_fields[1]);
	#$client_ip = ($logfile_fields[2]);
	$url = ($logfile_fields[4]);
	$reason = ($logfile_fields[5]);
	@hit_date = split /\./, $date, 3;
	$hit_year = ($hit_date[0]); $hit_month = ($hit_date[1]); $hit_day = ($hit_date[2]);

	# Uncomment the following for hostname resolution
	#&get_hostname();
	if ($tr_alternate == 1) {
	    print "<tr bgcolor=b7b7b7>";
	    $tr_alternate = $tr_alternate - 2;
	}

	print "<td>$hit_month\/$hit_day\/$hit_year $time</td>\n";
	#print "<td>$client_ip</td>\n";

	# Uncomment the following for hostname resolution
	#print "<td>$pc_name</td>";
	print "<td>$url</td>\n";
	print "<td>$reason</td></tr>\n";
    }
}

# Nice little "footer" thingy
print "</table>\n";
print "</body></html>\n";

close (LOG_FILE);
close (HTML);

# Return the '*.html' file to the state we found it ....err, sorta.
select STDOUT;

# Dansgurdian already takes care of rotating
# # Check to see if "access.log" needs rotating, and do so if it does
# if (-M $log_file > 0.98) {
#     move ("$log_file", "$log_file.$month.$day.$year");
# }
# 
# # Groom directory of files older than 14 days if more than 14 files exist
# @dir_contents = glob ("$parser_output_dir");
# $num_files = @dir_contents;
# if ($num_files > 14) {
#     foreach $file (@dir_contents) {
# 	if (-M $file > 14) {
# 	    unlink("$file");
# 	}
#     }
# }

# Get a client computer's name
#sub get_hostname
#{
#	use Socket;
#	$ipaddr = inet_aton("$client_ip");
#	$pc_name = gethostbyaddr ($ipaddr, AF_INET);
#}

# $
# $	(Cougaindex version 2.6)
# $
# $	Simple script with no administration which is used as a default indexer, allows you
# $	to customize the layout of directories say for downloading purposes... because the
# $	files are generated on the fly it will only display the contents of what is in the
# $	specific directory.
# $
# $	This code is distributed in the hope that is will be useful but WITHOUT ANY
# $	WARRANTY. ALL WARRANTIES, EXPRESSED OR IMPLIED ARE HEREBY DISCLAMED. This includes
# $	but isn't limited to warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# $	PURPOSE.  The RESELLING of this code is STRICTLY PROHIBITED.
# $
# $	$Revision: 2.6
# $	$Author: Paul Williams
# $	$Email: paul@rainbow.nwnet.co.uk
# $	$URL: http://www.cougasoft.org/
# $	$Created: 05/12/1996 11:01
# $	$Last Modified: 01/04/2000 18:12
# $
# $	Copyright 1996, 1997, 1998, 1999, 2000 Cougasoft.  All rights reserved.
# $

# Open 'index.html' for writing, and select it to send output to
$index_html=("$destdir/index.html");
open( INDEX_HTML, ">", "$index_html" );
select INDEX_HTML;

	# $ The http location of the graphics, so the below variable when viewed on
        # $ your website will look like:  http://www.yourdomain.com/cougaindex/icons
	$imgloc	 = '/usr/share/webcontentcontrol/webcontentcontrol_GUI/icon/';
	$imgloc  .= "/" if ($imgloc !~ /\/$/);

        # $ Set the colours for the script to output here, I'm assuming you have either
        # $ a working or rough knowledge of perl so editing them by hand shouldn't be
        # $ a problem.  If you get errors and have edited it, double check the variables.
	$BODYTEXT	= '#000000';
	$BGCOLOR	= '#C0C0C0';
	$LINK		= '#004080';
	$VLINK		= '#0000A0';
	$ALINK		= '#0000A0';
	$TBCOLOR	= '#BABABA';    # $ Colour of the table.
        $BACKGROUNDGIF  = '';

# 	$ENV{'SCRIPT_NAME'} =~ /(.*)\/index[\w\W]+.[\w\W]+$/;
# 	print STDOUT $ENV{'SCRIPT_NAME'};

#
# $--------------------------------------------------------------------------------------------
#
#  M  A  I  N	  P  R  O  G  R  A  M
#
# $--------------------------------------------------------------------------------------------
#

        print "Content-type: text/html\n\n" .
	    # $ Please don't change this, people also seem to like to alter this which is
	    # $ also a little dis-heartening, especially as its hidden inside the html output.
	    "<!-- © Copyright 1999 Cougasoft [paul\@rainbow.nwnet.co.uk] -->\n" .
            "<HTML>\n" .
            "<HEAD>\n" .
            "   <TITLE>Cougaindex - Dansguardian logs</TITLE>\n";
	print "<BODY TEXT='$BODYTEXT' BGCOLOR='$BGCOLOR' LINK='$LINK' VLINK='$VLINK' ALINK='$ALINK' BACKGROUND='$BACKGROUNDGIF'>\n";

	print "<TABLE WIDTH='100%' BGCOLOR='$TBCOLOR' BORDER=0 >\n" .
            "<TR>\n" .
	    "<TD WIDTH='1%'></TD>\n" .
	    "<TD WIDTH='90%'><I><FONT SIZE=-1>Filename</FONT></FONT></I></TD>\n" .
	    "<TD ALIGN=CENTER WIDTH='5%'><I><FONT SIZE=-1>Size</FONT></FONT></I></TD>\n" .
	    "<TD>&nbsp;</TD>\n";
	print "</TR>\n";

	print "<TR>\n" .
	    "<TD WIDTH='1%'><IMG SRC='$imgloc/dir.gif'></TD>\n" .
	    "<TD WIDTH='90%'><FONT SIZE=-1><A HREF='../'>..</A></FONT></TD>\n" .
	    "<TD ALIGN=CENTER><FONT SIZE=-1>-</FONT></TD>\n" .
	    "<TD><I><FONT SIZE=-1>Higher Dir</FONT></I></TD>\n";
	print "</TR>\n";

	opendir(DIR, $destdir);
	@dir = sort(readdir(DIR));

	foreach (@dir)
	{
                next if (/^[\.]/);
                next if (/index[\w\W]+.[\.cgi\.pl]+$/i);

	        if (-d) { push(@directory, $_); }
	        elsif (/readme/i) { $read = $_; }
	        else { push(@file, $_); }

	}
	closedir(DIR);

        foreach (@directory)
        {
	    	print "<TR>\n" .
			"<TD WIDTH='1%'><IMG SRC='$imgloc/dir.gif'></TD>\n" .
			"<TD WIDTH='90%'><FONT SIZE=-1><A HREF='$_/'>$_</A></FONT></TD>\n" .
			"<TD ALIGN=CENTER><FONT SIZE=-1>-</FONT></TD>\n" .
			"<TD><FONT SIZE=-1><I>Dir</I></FONT></TD>\n";
		print "</TR>\n";
        }

        foreach (@file)
        {
           if (/\.txt\b|\.html\b|\.htm\b|\.shtml\b|\.doc\b|\.rtf\b|\.dot\b|\.wri\b/i) {
	    $img = "html.gif";
	    $type= "Document";
	   } elsif (/\.gif\b|\.jpg\b|\.bmp\b|\.jpeg\b|\.jpe\b|\.jfif\b|\.pjprg\b|\.pjp\b|\.psd\b|\.xbm\b|\.ief\b|\.png\b|\.pcd\b|\.rgb\b|\.ppm\b|\.pgm\b|\.pbm\b|\.pnm\b|\.xwd\b|\.xpm\b|\.xbm\b|\.ras\b|\.tiff\b|\.tif\b/i) {
	      $img = "image.gif";
	      $type= "Image";
	     } elsif (/\.pl\b|\.cgi\b|\.c\b|\.sh\b/i) {
		$img = "script.gif";
		$type= "Script";
	       } elsif (/\.exe\b|\.com\b|\.bat\b|\.bin\b/i) {
		  $img = "exe.gif";
		  $type= "Executable";
		 } elsif (/\.zip\b|\.tar\b|\.gz\b|\.z\b|\.lzh\b|\.arj\b|\.arc\b|\.taz\b|\.shar\b|\.gtar\b|\.hqx\b/i) {
		    $img = "compress.gif";
		    $type= "Compressed";
		   } elsif (/\.wav\b|\.midi\b|\.mid\b|\.au\b|\.snd\b|\.aif\b|\.aiff\b|\.aifc\b|\.rmi\b|\.ra\b|\.ram\b|\.mp2\b|\.mpa\b|\.abs\n|\.mpega\b/i) {
		      $img = "audio.gif";
		      $type= "Sound";
		     } elsif (/\.mov\b|\.qt\b|\.moov\b|\.mpv2\b|\.mp2v\b|\.mpeg\b|\.mpg\b|\.mpe\b|\.mpv\b|\.vbs\b|\.mpegv\b/i) {
			$img = "move.gif";
			$type= "Video";
		       } else {
			  $img = "anon.gif";
			  $type = "?";
		         }
#            $SIZE = (-s);
           $SIZE = -s "$destdir/$_";
# 	   $SIZE=$SIZEe;
	   print STDOUT "$destdir/$_\n";
# 	    $SIZE = 1024;
	   printf("<TR>\n" .
		"<TD WIDTH='1%%'><IMG SRC='%s%s'></TD>\n" .
		"<TD WIDTH='90%%'><FONT SIZE=-1><A HREF='%s'>%s</A></FONT></TD>\n" .
		"<TD ALIGN=CENTER><FONT SIZE=-1> %d kb</FONT></TD>\n" .
		"<TD><I><FONT SIZE=-1>%s</FONT></I></TD>\n", $imgloc, $img, $_, $_,
                 int($SIZE / 1024), $type);
	   printf("</TR>\n");
        }

	print "</TABLE>\n";

        if ($read)
        {
                open (README, "$read");
                  print "<PRE>";
                    while (<README>)
                     {
                        print ;
                     }
                  print "</PRE>";
                close (README);
        }

	print "</BODY>\n";
	print "</HTML>\n";

#
# $--------------------------------------------------------------------------------------------
#  ===========================================================================================
# $--------------------------------------------------------------------------------------------
#
close(INDEX_HTML);
